use actix_web::{web, HttpResponse, Responder};
use rinja::Template;
use sqlx::PgPool;

use crate::{
    endpoints::{events::NewEventTemplate, NaiveDateQuery},
    models::{Location, Role, User},
    utils::ApplicationError,
};

#[actix_web::get("/events/new")]
pub async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    query: web::Query<NaiveDateQuery>,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::Admin && user.role != Role::AreaManager {
        return Err(ApplicationError::Unauthorized);
    }

    let locations = if user.role == Role::Admin {
        Location::read_all_including_area(pool.get_ref()).await?
    } else {
        Location::read_by_area(pool.get_ref(), user.area_id).await?
    };

    let template = NewEventTemplate {
        user: user.into_inner(),
        date: query.date,
        locations,
        event: None,
        amount_of_planned_posten: 0,
        is_fuehrungsassistent_planned: false,
        is_wachhabender_planned: false
    };

    Ok(HttpResponse::Ok().body(template.render()?))
}
